/*
 * This code is in public domain
 */

#ifndef _WIN_GETTIMEOFDAY_H_
#define _WIN_GETTIMEOFDAY_H_

#include "libmesh/libmesh_config.h"

#ifndef LIBMESH_HAVE_GETTIMEOFDAY
#ifdef _MSC_VER
#define NOMINMAX
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdint.h> // portable: uint64_t   MSVC: __int64

// MSVC defines this in winsock2.h!?
typedef struct timeval {
    long tv_sec;
    long tv_usec;
} timeval;

inline int gettimeofday(struct timeval * tp, struct timezone * tzp)
{
    // This magic number is the number of 100 nanosecond intervals since January 1, 1601 (UTC)
    // until 00:00:00 January 1, 1970
    static const uint64_t EPOCH = ((uint64_t) 116444736000000000ULL);

    SYSTEMTIME  system_time;
    FILETIME    file_time;
    uint64_t    time;

    GetSystemTime(&system_time);
    SystemTimeToFileTime(&system_time, &file_time);
    time =  ((uint64_t)file_time.dwLowDateTime );
    time += ((uint64_t)file_time.dwHighDateTime) << 32;

    tp->tv_sec  = (long) ((time - EPOCH) / 10000000L);
    tp->tv_usec = (long) (system_time.wMilliseconds * 1000);
    return 0;
}
#else // _MSC_VER
#error "gettimeofday() is not implemented"
#endif // _MSC_VER
#endif // !LIBMESH_HAVE_GETTIMEOFDAY

#endif // _WIN_GETTIMEOFDAY_H_
